/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.builder;

import static java.util.Objects.requireNonNull;
import static org.fortiss.tooling.spiderchart.style.FillStyle.GRADIENT_VERTICAL;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.fortiss.tooling.spiderchart.SpiderChartOld;
import org.fortiss.tooling.spiderchart.plotter.SpiderChartLegend;
import org.fortiss.tooling.spiderchart.plotter.SpiderChartPlotter;
import org.fortiss.tooling.spiderchart.plotter.SpiderChartTitle;
import org.fortiss.tooling.spiderchart.style.FillStyle;
import org.fortiss.tooling.spiderchart.swt.SpiderChartViewer;

/**
 * Used to build the complete spider chart
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class SpiderChartBuilder {
	/** The Legend */
	private SpiderChartLegend legend;

	/** The plotter */
	private SpiderChartPlotter plotter;

	/** The title */
	private SpiderChartTitle title;

	/** Prepares the legend */
	public SpiderChartBuilder(SpiderChartTitle title, SpiderChartLegend legend,
			SpiderChartPlotter plotter) {
		requireNonNull(title);
		this.title = title;
		this.legend = legend;
		requireNonNull(plotter);
		this.plotter = plotter;
	}

	/** Builds the viewer */
	public SpiderChartViewer createViewer(Composite parent) {
		requireNonNull(parent);
		// Some chart related default configurations
		SpiderChartOld chart = new SpiderChartOld(title, plotter);
		chart.setBackStyle(new FillStyle(SWT.COLOR_YELLOW));
		chart.setBackgroundCanvasColor(SWT.COLOR_WHITE);
		chart.getBackStyle().setGradientType(GRADIENT_VERTICAL);
		chart.setLegend(legend);
		chart.setActivateSelection(true);

		SpiderChartViewer chartViewer = new SpiderChartViewer(parent, SWT.NONE);
		chartViewer.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		chartViewer.setBounds(parent.getShell().getClientArea().x, parent.getShell()
				.getClientArea().y, parent.getShell().getClientArea().width, parent.getShell()
				.getClientArea().height - 10);
		chartViewer.setChart(chart);
		return chartViewer;
	}
}
