/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.example;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.fortiss.tooling.spiderchart.builder.SpiderChartBuilder;
import org.fortiss.tooling.spiderchart.model.Axis;
import org.fortiss.tooling.spiderchart.model.DataSeries;
import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.plotter.SpiderChartLegend;
import org.fortiss.tooling.spiderchart.plotter.SpiderChartPlotter;
import org.fortiss.tooling.spiderchart.plotter.SpiderChartTitle;
import org.fortiss.tooling.spiderchart.style.AxisStyle;
import org.fortiss.tooling.spiderchart.swt.SpiderChartViewer;

/**
 * Example application for the spider chart widget.
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
@SuppressWarnings("javadoc")
public final class Sample {

	// FIXME: enumeration type not supported yet
	public enum Brand {
		COMMUNAL, INTERNATIONAL, LOCAL, OUT_OF_MARKET, STANDARD
	}

	private static SpiderChartViewer viewer;

	private static void buildSpiderChart(final Shell shell) {
		SpiderChart spiderChart = new SpiderChart();

		Axis battery = new Axis("Battery", 0.0, 5.0);
		Axis camera = new Axis("Camera", 0.0, 5.0);
		Axis display = new Axis("Display", 0.0, 5.0);
		Axis memory = new Axis("Memory", 0.0, 5.0);
		Axis brand = new Axis("Brand", 0.0, 5.0);
		Axis screen = new Axis("Screen", 0.0, 5.0);
		spiderChart.addAxis(battery);
		spiderChart.addAxis(camera);
		spiderChart.addAxis(display);
		spiderChart.addAxis(memory);
		spiderChart.addAxis(brand);
		spiderChart.addAxis(screen);

		DataSeries iPhoneData = new DataSeries("iPhone 6");
		iPhoneData.setPoint(battery, 4.0);
		iPhoneData.setPoint(camera, 3.5);
		iPhoneData.setPoint(display, 4.0);
		iPhoneData.setPoint(memory, 4.6);
		iPhoneData.setPoint(brand, 5.0);
		iPhoneData.setPoint(screen, 3.8);
		spiderChart.addData(iPhoneData);

		SpiderChartTitle title = new SpiderChartTitle("Smartphone Comparison Scale", spiderChart);
		title.setSwtForegroundColor(SWT.COLOR_DARK_BLUE);

		SpiderChartLegend legend = new SpiderChartLegend(spiderChart);
		legend.addItem(iPhoneData, SWT.COLOR_GREEN);
		legend.setOffset(320);
		legend.setVerticalLayout(true);
		legend.setSwtBackgroundColor(SWT.COLOR_WHITE);
		legend.setSwtForegroundColor(SWT.COLOR_BLUE);

		SpiderChartPlotter plotter = new SpiderChartPlotter(spiderChart);
		plotter.setAxisStyle(battery, AxisStyle.DEFAULT);
		plotter.setAxisStyle(camera, AxisStyle.DEFAULT);
		plotter.setAxisStyle(display, AxisStyle.DEFAULT);
		plotter.setAxisStyle(memory, AxisStyle.DEFAULT);
		plotter.setAxisStyle(brand, AxisStyle.DEFAULT);
		plotter.setAxisStyle(screen, AxisStyle.DEFAULT);

		SpiderChartBuilder scBuilder = new SpiderChartBuilder(title, legend, plotter);
		viewer = scBuilder.createViewer(shell);

		viewer.getChart().addSeq(iPhoneData);

		// Updating the chart with new parameters
		// Display.getDefault().asyncExec(() -> {
		// // changing values in runtime
		// final LineDataSeq iPhoneDataSequence =
		// LineDataSeq.of(iPhoneData.get(), 2.0, 4.2, 4.1, 42.8, 3.7,
		// Brand.INTERNATIONAL);
		// // Set the first sequence
		// viewer.getChart().getSpiderPlotter().setSeq(0, iPhoneDataSequence);
		//
		// // changing axes in runtime
		// final AxesConfigurer configuration =
		// new AxesConfigurer.Builder().addAxis("Battery", 5, 0)
		// .addAxis("Screen", 5, 0).addAxis("Display", 5, 0)
		// .addAxis("Memory", 50, 0).addAxis("Sound", 5, 0)
		// .addAxis("Brand", Brand.class).build();
		//
		// final LineDataSeq nexusDataSequence =
		// LineDataSeq.of(nexusData.get(), 2.4, 3.2, 2.1, 23.8, 1.7, Brand.LOCAL);
		//
		// // Set the second sequence
		// viewer.getChart().getSpiderPlotter().setSeq(1, nexusDataSequence);
		// viewer.getChart().getSpiderPlotter().use(configuration);
		// viewer.getChart().getSpiderPlotter().setMarkScalesOnEveryAxis(true);
		// });
	}

	public static void main(final String[] args) {
		final Display display = new Display();
		final Shell shell = new Shell(display, SWT.DIALOG_TRIM);
		shell.setSize(800, 750);

		buildSpiderChart(shell);
		shell.open();

		while(!shell.isDisposed()) {
			if(!display.readAndDispatch()) {
				display.sleep();
			}
		}

		viewer.getChart().dispose();

		display.dispose();
	}
}
