/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.gc;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.spiderchart.gc.swt.SpiderChartSwtGraphics;
import org.fortiss.tooling.spiderchart.gc.swt.SpiderChartSwtImage;

/**
 * Represents an abstract factory for graphics to be used in the chart
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class AbstractGraphicsSupplier {

	/** Creates an image */
	public static AbstractChartImage createImage(final int w, final int h) {
		return new SpiderChartSwtImage(w, h);
	}

	/** creates transparent image */
	public static AbstractChartImage createTransparentImage(final int w, final int h,
			Color transparent) {
		return new SpiderChartSwtImage(w, h, transparent);
	}

	/** Returns provided RGB color */
	public static Color getColor(final int red, final int green, final int blue) {
		return new Color(Display.getCurrent(), red, green, blue);
	}

	/** returns the font based the the provided font and style */
	public static Font getFont(Fonts fonts, int style, int size) {
		return new Font(Display.getCurrent(), fonts.getFontName(), size, style);
	}

	/** returns the graphics */
	public static SpiderChartSwtGraphics getGraphics(final Object o) {
		return new SpiderChartSwtGraphics(o);
	}

	/** returns the image */
	public static AbstractChartImage getImage(final Object o) {
		try {
			return new SpiderChartSwtImage(o);
		} catch(final Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/** starts the paint in UI thread */
	public static void startUiThread(final Runnable r) {
		Display.getCurrent().syncExec(r);
	}

	/**
	 * Constructor
	 */
	private AbstractGraphicsSupplier() {
	}
}
