/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.gc.swt;

import static org.fortiss.tooling.spiderchart.gc.AbstractGraphicsSupplier.getImage;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.spiderchart.gc.AbstractChartImage;

/**
 * Represents a chart graphics to be used in SWT
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class SpiderChartSwtGraphics {
	/** */
	public static int ROTATE_CENTER = 0;
	/** */
	public static int ROTATE_LEFTTOP = 1;
	/** */
	public static int STROKE_DASHED = 3;
	/** */
	public static int STROKE_DOTTED = 2;
	/** */
	public static int STROKE_NORMAL = 1;
	/** */
	protected int lineStyle = STROKE_NORMAL;
	/** */
	protected int lineWidth = 1;
	/** */
	private float alphaValue = 1.0F;
	/** */
	private GC graphics;
	/** */
	private Image imageForTransparentFilling = null;
	/** */
	private Image srcImage;
	/** */
	private Color transparent = null;
	/** */
	protected AbstractChartImage textureImage = null;

	/** Constructor */
	public SpiderChartSwtGraphics(final Object g) {
		// FIXME: wild cast
		this.graphics = (GC)g;
	}

	/** Creates fade area */
	public void createFadeArea(Color colorFrom, Color colorTo, int x, int y, int w, int h,
			boolean vertical) {
		Color fg = graphics.getForeground();
		graphics.setForeground(colorFrom);
		Color bg = graphics.getBackground();
		graphics.setBackground(colorTo);

		graphics.fillGradientRectangle(x, y, w, h, vertical);

		graphics.setBackground(bg);
		graphics.setForeground(fg);
	}

	/** disposes the resource */
	public void dispose() {
		if(graphics != null) {
			graphics.dispose();
		}
		graphics = null;
	}

	/** draws the arc as provided as inputs */
	public void drawArc(int x, int y, int w, int h, int a1, int a2) {
		graphics.drawArc(x, y, w, h, a1, a2);
	}

	/** draws the image */
	public void drawImage(AbstractChartImage image, int x, int y) {
		if(image == null) {
			return;
		}
		if(((SpiderChartSwtImage)image).getImage() == null) {
			return;
		}
		graphics.drawImage(((SpiderChartSwtImage)image).getImage(), x, y);
	}

	/** draws the image */
	public void drawImage(AbstractChartImage image, int x1Dest, int y1Dest, int x2Dest, int y2Dest,
			int x1Source, int y1Source, int x2Source, int y2Source) {
		if(image == null) {
			return;
		}
		if(((SpiderChartSwtImage)image).getImage() == null) {
			return;
		}
		graphics.drawImage(((SpiderChartSwtImage)image).getImage(), x1Source, y1Source, x2Source -
				x1Source, y2Source - y1Source, x1Dest, y1Dest, x2Dest - x1Dest, y2Dest - y1Dest);
	}

	/** draws the line */
	public void drawLine(int x1, int y1, int x2, int y2) {
		graphics.drawLine(x1, y1, x2, y2);
	}

	public void drawLineWithStyle(int x1, int y1, int x2, int y2) {
		if(this.lineStyle == STROKE_NORMAL) {
			this.drawSimpleLine(x1, y1, x2, y2);
		} else {
			int segment = 10;
			int segmentspace = 10;
			if(this.lineStyle == STROKE_DOTTED) {
				segment = 1;
				segmentspace = 5;
			}
			int h = Math.abs(y2 - y1);
			int w = Math.abs(x2 - x1);
			if(h == 0 && w == 0) {
				return;
			}
			double hipo = Math.sqrt(h * h + w * w);

			double Cos = w / hipo;
			double Sin = h / hipo;

			int Xslope = 1;
			int Yslope = 1;
			if(x2 < x1) {
				Xslope = -1;
			}
			if(y2 < y1) {
				Yslope = -1;
			}
			int subx1 = 0;
			int suby1 = 0;
			int subx2 = 0;
			int suby2 = 0;
			int subsegment = 0;
			for(;;) {
				suby2 = (int)(Sin * (subsegment + segment)) * Yslope;
				subx2 = (int)(Cos * (subsegment + segment)) * Xslope;
				suby1 = (int)(Sin * subsegment) * Yslope;
				subx1 = (int)(Cos * subsegment) * Xslope;
				if(w < Math.abs(subx1) || h < Math.abs(suby1)) {
					break;
				}
				if(w < Math.abs(subx2)) {
					subx2 = w * Xslope;
				}
				if(h < Math.abs(suby2)) {
					suby2 = h * Yslope;
				}
				this.drawSimpleLine(x1 + subx1, y1 + suby1, x1 + subx2, y1 + suby2);

				subsegment = subsegment + segment + segmentspace;
			}
		}
	}

	/** draws the simple line */
	protected void drawSimpleLine(int x1, int y1, int x2, int y2) {
		int pixelsWidth = 1;

		pixelsWidth = this.lineWidth * 1;
		if(pixelsWidth < 1) {
			pixelsWidth = 1;
		}
		this.drawLine(x1, y1, x2, y2);
		if(pixelsWidth == 1) {
			return;
		}
		if(pixelsWidth > 1) {
			int xwidth = 0;
			int ywidth = 0;

			int h = Math.abs(y2 - y1);
			int w = Math.abs(x2 - x1);

			double hipo = Math.sqrt(h * h + w * w);

			double Cos = w / hipo;

			xwidth = 1;
			ywidth = 0;
			if(Cos > Math.cos(1.0471666666666668D)) {
				xwidth = 0;
				ywidth = 1;
			}
			if(Cos > Math.cos(0.5235833333333334D)) {
				xwidth = 0;
				ywidth = 1;
			}
			int side = 1;
			int distanceToCenter = 0;
			for(int i = 2; i <= pixelsWidth; i++) {
				if(side == 1) {
					distanceToCenter++;
				}
				this.drawLine(x1 + side * xwidth * distanceToCenter, y1 + side * ywidth *
						distanceToCenter, x2 + side * xwidth * distanceToCenter, y2 + side *
						ywidth * distanceToCenter);
				side *= -1;
			}
		}
	}

	/** draws the polygon */
	public void drawPolygon(int[] x1, int[] y1, int count) {
		int[] points = new int[count * 2];
		int j = 0;
		for(int i = 0; i < count; i++) {
			points[j++] = x1[i];
			points[j++] = y1[i];
		}
		this.graphics.drawPolygon(points);
	}

	/** draws the rectangle */
	public void drawRect(int x1, int y1, int w, int h) {
		this.drawLineWithStyle(x1, y1, x1, y1 + h);
		this.drawLineWithStyle(x1, y1, x1 + w, y1);
		this.drawLineWithStyle(x1, y1 + h, x1 + w, y1 + h);
		this.drawLineWithStyle(x1 + w, y1, x1 + w, y1 + h);
	}

	/** draws the rectangle with round corners */
	public void drawRoundRect(int x1, int y1, int w, int h) {
		this.drawRect(x1, y1, w, h); // FIXME
	}

	/** fills the rectangle with round corners */
	public void fillRoundRect(int x1, int y1, int w, int h, Color color) {
		this.fillRect(x1, y1, w, h, color); // FIXME
	}

	/** draws the text as rotated */
	public boolean drawRotatedText(Font descFont, Color descColor, String txt, int angle, int x,
			int y, boolean b) {
		// this.setFont(descFont);
		// this.setColor(descColor);

		int h = this.getFontHeight();
		int w = this.getFontWidth(txt);
		int size = w;
		int toCenterX = 0;
		int toCenterY = 0;
		if(h > w) {
			size = h;
			toCenterX = (size - w) / 2;
		} else {
			toCenterY = (size - h) / 2;
		}
		Image tmpImage = new Image(Display.getCurrent(), size, size);
		ImageData imageData = tmpImage.getImageData();
		tmpImage.dispose();

		Color transparent = null;
		if(descColor.getRed() == 255 && descColor.getBlue() == 255 && descColor.getGreen() == 255) {
			transparent = new Color(Display.getCurrent(), 0, 0, 0);
			imageData.transparentPixel = imageData.palette.getPixel(new RGB(0, 0, 0));
		} else {
			transparent = new Color(Display.getCurrent(), 255, 255, 255);
			imageData.transparentPixel = imageData.palette.getPixel(new RGB(255, 255, 255));
		}
		tmpImage = new Image(Display.getCurrent(), imageData);

		GC g = new GC(tmpImage);
		Color c = descColor;
		Font f = descFont;

		g.setForeground(transparent);
		g.setBackground(transparent);
		g.fillRectangle(0, 0, size, size);

		g.setFont(f);
		g.setForeground(c);
		g.drawText(txt, toCenterX, toCenterY, true);

		g.dispose();
		c.dispose();
		f.dispose();

		AbstractChartImage tmpChartImage = getImage(tmpImage);
		this.paintRotatedImage(tmpChartImage, angle, x - (w - h) / 2, y + 4, ROTATE_CENTER);

		tmpChartImage.dispose();
		tmpImage.dispose();

		return true;
	}

	public void drawText(String s, int x, int y, Color color, Font font) {
		Font oldFont = graphics.getFont();
		graphics.setFont(font);
		drawText(s, x, y, color);
		graphics.setFont(oldFont);
	}

	public void drawText(String s, int x, int y, Color color) {
		Color oldColor = graphics.getForeground();
		graphics.setForeground(color);
		graphics.drawString(s, x, y - graphics.getFontMetrics().getHeight(), true);
		graphics.setForeground(oldColor);
	}

	public void fillArc(int x, int y, int w, int h, int a1, int a2, Color color) {
		GC g = this.getGraphicForFilling(color);
		g.fillArc(x, y, w, h, a1, a2);
		this.processFilling(g);
	}

	public void fillPolygon(int[] x1, int[] y1, int count, Color color) {
		int[] points = new int[count * 2];
		int j = 0;
		for(int i = 0; i < count; i++) {
			points[j++] = x1[i];
			points[j++] = y1[i];
		}
		GC g = this.getGraphicForFilling(color);
		g.fillPolygon(points);
		this.processFilling(g);
	}

	public void fillRect(int x1, int y1, int w, int h, Color color) {
		GC g = this.getGraphicForFilling(color);
		g.fillRectangle(new Rectangle(x1, y1, w, h));
		this.processFilling(g);
	}

	public Object getAlphaComposite() {
		return new Float(this.alphaValue);
	}

	/**
	 * @return alpha value
	 */
	public float getAlphaValue() {
		return this.alphaValue;
	}

	/** Getter for font height */
	public int getFontHeight() {
		return graphics.getFontMetrics().getHeight();
	}

	public int getFontHeight(Font font) {
		Font old = graphics.getFont();
		graphics.setFont(font);
		int result = graphics.getFontMetrics().getHeight();
		graphics.setFont(old);
		return result;
	}

	public int getFontWidth(Font font, String s) {
		Font old = graphics.getFont();
		graphics.setFont(font);
		int result = graphics.textExtent(s).x;
		graphics.setFont(old);
		return result;
	}

	/**  */
	protected GC getGraphicForFilling(Color currentColor) {
		if(alphaValue >= 1.0F || srcImage == null) {
			imageForTransparentFilling = null;
			return graphics;
		}
		transparent = null;
		if(currentColor.getRed() == 255 && currentColor.getBlue() == 255 &&
				currentColor.getGreen() == 255) {
			transparent = Display.getCurrent().getSystemColor(SWT.COLOR_BLACK);
		} else {
			transparent = Display.getCurrent().getSystemColor(SWT.COLOR_WHITE);
		}
		RGB[] rgbs = new RGB[256];
		rgbs[0] = transparent.getRGB();
		for(int i = 1; i <= 255; i++) {
			rgbs[i] = currentColor.getRGB();
		}
		ImageData imageData =
				new ImageData(srcImage.getBounds().width, srcImage.getBounds().height,
						srcImage.getImageData().depth, srcImage.getImageData().palette);

		imageForTransparentFilling = new Image(Display.getCurrent(), imageData);

		GC g = new GC(imageForTransparentFilling);

		g.setForeground(transparent);
		g.setBackground(transparent);
		g.fillRectangle(0, 0, srcImage.getBounds().width, srcImage.getBounds().height);

		g.setForeground(currentColor);
		g.setBackground(currentColor);

		return g;
	}

	/**  */
	public GC getGraphics() {
		return graphics;
	}

	/**  */
	public Image getImageForTransparentFilling() {
		return imageForTransparentFilling;
	}

	/**  */
	public Image getSrcImage() {
		return srcImage;
	}

	/**  */
	public Color getTransparent() {
		return transparent;
	}

	public void paintRotatedImage(AbstractChartImage srcImage, int angle, int x, int y,
			int alignment) {
		Image srcSwtImage = ((SpiderChartSwtImage)srcImage).getImage();
		int size = srcSwtImage.getImageData().width;
		int h = srcSwtImage.getImageData().height;
		int w = srcSwtImage.getImageData().width;
		int originalH = h;
		if(h != w) {
			srcImage = ((SpiderChartSwtImage)srcImage).forRotation();
			srcSwtImage = ((SpiderChartSwtImage)srcImage).getImage();

			size = srcSwtImage.getImageData().width;
			h = srcSwtImage.getImageData().height;
			w = srcSwtImage.getImageData().width;
		}
		ImageData srcData = srcSwtImage.getImageData();
		ImageData destData = new ImageData(size, size, srcData.depth, srcData.palette);
		destData.transparentPixel = srcData.transparentPixel;
		if(destData.transparentPixel != -1) {
			for(int i = 0; i < size; i++) {
				for(int j = 0; j < size; j++) {
					destData.setPixel(i, j, destData.transparentPixel);
				}
			}
		}
		for(int i = 0; i < size; i++) {
			for(int j = 0; j < size; j++) {
				int x2 = rotateX(i - size / 2, j - size / 2, -angle) + size / 2;
				int y2 = rotateY(i - size / 2, j - size / 2, -angle) + size / 2;

				if(x2 < size && y2 < size && x2 >= 0 && y2 >= 0) {
					destData.setPixel(i, j, srcData.getPixel(x2, y2));
				}
			}
		}
		Image destImage = new Image(Display.getCurrent(), destData);
		if(alignment == ROTATE_LEFTTOP) {
			if(angle == 90) {
				graphics.drawImage(destImage, x - (w - originalH), y);
			} else if(angle == -90) {
				graphics.drawImage(destImage, x, y);
			}
		} else if(angle == 90) {
			graphics.drawImage(destImage, x, y);
		} else if(angle == -90) {
			graphics.drawImage(destImage, x, y);
		}
		destImage.dispose();
	}

	/** */
	protected void processFilling(GC g) {
		if(g != graphics) {
			g.dispose();

			ImageData imageData = imageForTransparentFilling.getImageData();
			for(int i = 0; i < srcImage.getBounds().width; i++) {
				for(int j = 0; j < srcImage.getBounds().height; j++) {
					RGB rgb = imageData.palette.getRGB(imageData.getPixel(i, j));
					if(rgb.red == transparent.getRed() && rgb.green == transparent.getGreen() &&
							rgb.blue == transparent.getBlue()) {
						imageData.setAlpha(i, j, 0);
					} else {
						imageData.setAlpha(i, j, (int)(alphaValue * 255.0F));
					}
				}
			}
			transparent.dispose();
			imageForTransparentFilling.dispose();

			imageForTransparentFilling = new Image(Display.getCurrent(), imageData);

			graphics.drawImage(imageForTransparentFilling, 0, 0);
			imageForTransparentFilling.dispose();
			imageForTransparentFilling = null;
		}
	}

	/** */
	private int rotateX(int x, int y, int angle) {
		if(angle == 90) {
			return -y;
		}
		if(angle == -90) {
			return y;
		}
		return (int)(x * Math.cos(Math.toRadians(angle)) - y * Math.sin(Math.toRadians(angle)));
	}

	/** */
	private int rotateY(int x, int y, int angle) {
		if(angle == 90) {
			return x * 1;
		}
		if(angle == -90) {
			return x * -1;
		}
		return (int)(x * Math.sin(Math.toRadians(angle)) + y * Math.cos(Math.toRadians(angle)));
	}

	public void setAlpha(float a) {
		alphaValue = a;
	}

	public void setAlphaComposite(Object a) {
		if(a instanceof Float) {
			alphaValue = ((Float)a).floatValue();
		}
	}

	/**  */
	public void setAlphaValue(float alphaValue) {
		this.alphaValue = alphaValue;
	}

	/** getter for the ont width */
	public int getFontWidth(String s) {
		return getFontWidth(null, s);
	}

	/**  */
	public void setGraphics(GC graphics) {
		this.graphics = graphics;
	}

	/**  */
	public void setImageForTransparentFilling(Image imageForTransparentFilling) {
		this.imageForTransparentFilling = imageForTransparentFilling;
	}

	public void setLineStyle(int style) {
		this.lineStyle = style;
	}

	public void setLineWidth(int w) {
		this.lineWidth = w;
	}

	/**  */
	public void setSrcImage(Image srcImage) {
		this.srcImage = srcImage;
	}

	/**  */
	public void setTransparent(Color transparent) {
		this.transparent = transparent;
	}

	/** setter for texture */
	public void setTexture(AbstractChartImage image) {
		this.textureImage = image;
	}
}
