/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.plotter;

import static java.util.Objects.requireNonNull;
import static org.fortiss.tooling.spiderchart.gc.Fonts.VERDANA;
import static org.fortiss.tooling.spiderchart.util.ChartUtil.enumConstants;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.spiderchart.gc.AbstractChartImage;
import org.fortiss.tooling.spiderchart.gc.Polygon;
import org.fortiss.tooling.spiderchart.gc.swt.SpiderChartSwtGraphics;
import org.fortiss.tooling.spiderchart.model.Axis;
import org.fortiss.tooling.spiderchart.model.DataSeries;
import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.scale.SpiderChartScale;
import org.fortiss.tooling.spiderchart.sequence.DataSequence;
import org.fortiss.tooling.spiderchart.sequence.LineDataSequence;
import org.fortiss.tooling.spiderchart.style.AxisStyle;
import org.fortiss.tooling.spiderchart.style.FillStyle;
import org.fortiss.tooling.spiderchart.style.LineStyle;

/**
 * Class for drawing the spider chart.
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class SpiderChartPlotter extends SpiderChartComponentBase {

	/** The map for style information of axes. */
	private Map<Axis, AxisStyle> axisStyle = new HashMap<Axis, AxisStyle>();
	/** The map for style information of data series. */
	private Map<DataSeries, LineStyle> dataSeriesStyle = new HashMap<DataSeries, LineStyle>();
	/** Spider Chart Background Image */
	private AbstractChartImage backgroundImage;
	/** The relative chart margin. The default is 10%. */
	private double relativeMargin = 0.9;

	/** depth of the component */
	private int depth = 0;

	/** The list of data sequences. */
	protected List<DataSequence> data = new ArrayList<DataSequence>();

	/** Adds sequence to the plotter */
	public void addSeq(DataSeries data) {
		replaceSeq(-1, data);
	}

	/** calculates the max values in the provided scale */
	private void calculateMax(final SpiderChartScale s, final double m) {
		if(!s.isExactMaxValue()) {
			s.setMax(m);
			return;
		}
		final double[] value = s.getPreferred_MaxMin_values();
		if(value != null && value.length > 0) {
			for(final double preferred_MaxMin_value : value) {
				if(preferred_MaxMin_value >= m) {
					s.setMax(preferred_MaxMin_value);

					break;
				}
			}
		}
	}

	/** calculates the min values in the provided scale */
	private void calculateMin(final SpiderChartScale s, final double m) {
		if(!s.isExactMinValue()) {
			s.setMin(m);
			return;
		}
		final double[] value = s.getPreferred_MaxMin_values();
		if(value != null && value.length > 0) {
			for(int j = value.length - 1; j > 0; j--) {
				if(value[j] <= m) {
					s.setMin(value[j]);

					break;
				}
			}
		}
	}

	/** replaces a sequence with the provided one */
	@SuppressWarnings("cast")
	public void replaceSeq(int p, DataSeries d) {
		SpiderChartScale tmpScaleX = xScale;
		// FIXME: ???
		// getActiveYScale();
		if(p >= data.size()) {
			return;
		}
		if(data.size() > 3) {
			return;
		}
		// FIXME: below
		// if(p == -1) {
		// data.add(new LineDataSequence(d.get, s));
		// } else {
		// data.set(p, s);
		// }
		// final boolean fixedLimits = false;
		// final boolean cumulative = false;
		// if(!(this instanceof SpiderChartPlotter)) {
		// for(int i = 0; i < s.getSize(); i++) {
		// if(s.getElementY(i) != null) {
		// final double XValue = ((Double)s.getElementX(i)).doubleValue();
		// double YValue = ((Double)s.getElementY(i)).doubleValue();
		// if(cumulative) {
		// YValue = 0.0D;
		// for(int si = 0; si < data.size(); si++) {
		// final DataSequence ser = data.get(si);
		// }
		// }
		// if(XValue >= tmpScaleX.getMax()) {
		// calculateMax(tmpScaleX, XValue);
		// }
		// if(XValue < tmpScaleX.getMin()) {
		// calculateMin(tmpScaleX, XValue);
		// }
		// if(!fixedLimits) {
		// if(YValue > yScale.getMax()) {
		// calculateMax(yScale, YValue);
		// }
		// if(YValue < yScale.getMin()) {
		// calculateMin(yScale, YValue);
		// }
		// }
		// }
		// }
		// }
	}

	/** visible height on the canvas */
	private int visibleHeight = 0;

	/** visible width on the canvas */
	private int visibleWidth = 0;

	/** x-axis scaling */
	private SpiderChartScale xScale;

	/** y-axis scaling */
	private SpiderChartScale yScale;
	/**
	 * axes names
	 */
	private String[] axesFactors;

	/**
	 * Axes Factor Color
	 */
	private int swtAxisFactorColor = SWT.COLOR_BLACK;

	/**
	 * Axis Factor Colors (in case you need to set different colors
	 * for different axes)
	 */
	private int[] swtAxisFactorColors;

	/**
	 * Axes Factor Text Font
	 */
	private FontData axisFactorFontData = new FontData(VERDANA.getFontName(), SWT.NORMAL, 11);

	/**
	 * Polygon Area Background Style
	 */
	private FillStyle backStyle;

	/**
	 * Border Style
	 */
	private LineStyle border = new LineStyle(0.2F, SWT.COLOR_BLACK, 1);

	/**
	 * Radius
	 */
	private double chartRadius = 0.9D;

	/**
	 * would be surrounded by an enclosing circle
	 */
	private boolean drawCircle = false;

	/**
	 * Font for Grid Label
	 */
	private FontData gridFontData;

	/**
	 * Font Color for Grid Label
	 */
	private int swtGridFontColor = SWT.COLOR_DARK_GRAY;

	/**
	 * Grid Style
	 */
	private LineStyle gridStyle;

	/** Mark Scales on Every Axis */
	private boolean markScalesOnEveryAxis = false;

	/**
	 * Scaling Factors (Maximum Values)
	 */
	private double[] maxScaleFactors;

	/**
	 * Scaling Factors (Minimum Values)
	 */
	private double[] minScaleFactors;

	/** */
	private Color[] pointColors = null;

	/** */
	private double[] pointColorScale = null;

	/**
	 * Scaling Divisions
	 */
	private int scalingDivisions = 5;

	/**
	 * Scaling Label Format
	 */
	private Object[] scalingLabelFormat;

	/** Constructor. */
	public SpiderChartPlotter(SpiderChart chart) {
		super(chart);
	}

	/**
	 * @return the axesFactors
	 */
	public String[] getAxesFactors() {
		return axesFactors;
	}

	/**
	 * @return the axisFactorColor
	 */
	public int getAxisFactorColor() {
		return swtAxisFactorColor;
	}

	/**
	 * @return the axisFactorColors
	 */
	public int[] getAxisFactorColors() {
		return swtAxisFactorColors;
	}

	/**
	 * @return the backStyle
	 */
	public FillStyle getBackStyle() {
		return backStyle;
	}

	/**
	 * @return the border
	 */
	public LineStyle getBorder() {
		return border;
	}

	/**
	 * @return the chartRadius
	 */
	public double getChartRadius() {
		return chartRadius;
	}

	/**
	 * @return the gridStyle
	 */
	public LineStyle getGridStyle() {
		return gridStyle;
	}

	/**
	 * @return the maxScaleFactors
	 */
	public double[] getMaxScaleFactors() {
		return maxScaleFactors;
	}

	/**
	 * @return the minScaleFactors
	 */
	public double[] getMinScaleFactors() {
		return minScaleFactors;
	}

	/**
	 * @return the pointColors
	 */
	public Color[] getPointColors() {
		return pointColors;
	}

	/**
	 * @return the pointColorScale
	 */
	public double[] getPointColorScale() {
		return pointColorScale;
	}

	/**
	 * @return the scalingDivisions
	 */
	public int getScalingDivisions() {
		return scalingDivisions;
	}

	/**
	 * @return the scalingLabelFormat
	 */
	public Object[] getScalingLabelFormat() {
		return scalingLabelFormat;
	}

	/**
	 * @return the drawCircle
	 */
	public boolean isDrawCircle() {
		return drawCircle;
	}

	/**
	 * @return the markScalesOnEveryAxis
	 */
	public boolean isMarkScalesOnEveryAxis() {
		return markScalesOnEveryAxis;
	}

	@SuppressWarnings({"unchecked", "rawtypes"})
	protected void plotSequence(SpiderChartSwtGraphics graphics, DataSequence seq,
			int sequenceNumber) {
		if(!(seq instanceof LineDataSequence)) {
			return;
		}
		LineDataSequence line = (LineDataSequence)seq;
		seq.getHotAreas().clear();
		int count = line.getyData().size();
		int[] xs = new int[count];
		int[] ys = new int[count];

		int radi = width;
		if(height < radi) {
			radi = height;
		}
		radi = (int)(radi * chartRadius);

		int toCenterX = (width - radi) / 2;
		int toCenterY = (height - radi) / 2;

		int PieCenterX = toCenterX + x + radi / 2;
		int PieCenterY = toCenterY + y + radi / 2;
		double angle;
		if(sequenceNumber == 0 || sequenceNumber >= data.size() - 1) {
			if(sequenceNumber == 0 && backStyle != null && drawCircle) {
				backStyle.drawArc(graphics, toCenterX + x, toCenterY + y, radi, radi, 0, 360);
			}
			if(sequenceNumber == 0 && backStyle != null && !drawCircle) {
				for(int i = 0; i < count; i++) {
					angle = 360.0D / count * i;

					angle += 90.0D;
					if(angle > 360.0D) {
						angle -= 360.0D;
					}
					double radian = 0.01745277777777778D * angle;
					double Sin = Math.sin(radian);
					double Cos = Math.cos(radian);
					int relativeY = (int)(Sin * (radi / 2));
					int relativeX = (int)(Cos * (radi / 2));
					relativeY *= -1;

					xs[i] = PieCenterX + relativeX;
					ys[i] = PieCenterY + relativeY;
				}
				backStyle.drawPolygon(graphics, xs, ys, (int)count);
			}
			if(sequenceNumber == data.size() - 1 && border != null) {
				if(drawCircle) {
					border.drawArc(graphics, toCenterX + x, toCenterY + y, radi, radi, 0, 360);
				}
				for(int i = 0; i < count; i++) {
					angle = 360.0D / count * i;

					angle += 90.0D;
					if(angle > 360.0D) {
						angle -= 360.0D;
					}
					double radian = 0.01745277777777778D * angle;
					double Sin = Math.sin(radian);
					double Cos = Math.cos(radian);
					int relativeY = (int)(Sin * (radi / 2));
					int relativeX = (int)(Cos * (radi / 2));
					relativeY *= -1;

					border.draw(graphics, PieCenterX, PieCenterY, PieCenterX + relativeX,
							PieCenterY + relativeY);
				}
			}
			if(sequenceNumber == 0 && axisFactorFontData != null) {
				Font ontheflyFont = new Font(Display.getCurrent(), axisFactorFontData);
				for(int i = 0; i < count; i++) {
					Color color = systemColor(swtAxisFactorColor);
					if(swtAxisFactorColors != null && swtAxisFactorColors.length > i) {
						color = systemColor(swtAxisFactorColors[i]);
					}
					angle = 360.0D / count * i;

					angle += 90.0D;
					if(angle > 360.0D) {
						angle -= 360.0D;
					}
					int tmpradi = (int)(radi * 1.1D / 2.0D);
					int correction = 0;
					if(angle > 120.0D && angle < 240.0D) {
						correction = graphics.getFontWidth(axesFactors[i]);
					}
					double radian = 0.01745277777777778D * angle;
					double sin = Math.sin(radian);
					double cos = Math.cos(radian);
					int relativeY = (int)(sin * tmpradi);
					int relativeX = (int)(cos * tmpradi);
					relativeY *= -1;
					if(axesFactors.length > i) {
						graphics.drawText(axesFactors[i], PieCenterX + relativeX - correction,
								PieCenterY + relativeY, color, ontheflyFont);
					}
				}
			}
		}
		for(int i = 0; i < count; i++) {
			angle = 360.0D / count * i;

			angle += 90.0D;
			if(angle > 360.0D) {
				angle -= 360.0D;
			}
			int tmpradi = 0;

			double min = 0.0D;
			double max = 100.0D;
			if(minScaleFactors.length >= i + 1) {
				min = minScaleFactors[i];
			}
			if(maxScaleFactors.length >= i + 1) {
				max = maxScaleFactors[i];
			}
			tmpradi =
					(int)((((Double)line.getElementY(i)).doubleValue() - min) * 100.0D / (max - min));
			tmpradi = tmpradi * radi / 100;

			double radian = 0.01745277777777778D * angle;
			double Sin = Math.sin(radian);
			double Cos = Math.cos(radian);
			int relativeY = (int)(Sin * (tmpradi / 2));
			int relativeX = (int)(Cos * (tmpradi / 2));
			relativeY *= -1;
			xs[i] = PieCenterX + relativeX;
			ys[i] = PieCenterY + relativeY;
		}
		if(line.getStyle() != null) {
			line.getStyle().drawPolygon(graphics, xs, ys, (int)count);
		}
		if(line.getFillStyle() != null) {
			line.getFillStyle().drawPolygon(graphics, xs, ys, (int)count);
		}
		int kl = 0;
		for(int i = 0; i < count; i++) {
			Polygon po = new Polygon();
			po.addPoint(xs[i] - 3, ys[i] - 3);
			po.addPoint(xs[i] - 3, ys[i] + 3);
			po.addPoint(xs[i] + 3, ys[i] + 3);
			po.addPoint(xs[i] + 3, ys[i] - 3);
			seq.getHotAreas().add(po);
			double YValue;
			if(line.isDrawPoint()) {
				Color color = systemColor(line.getPointColor());
				YValue = ((Double)line.getElementY(i)).doubleValue();
				if(pointColors != null && pointColorScale != null) {
					if(pointColors.length > 0) {
						color = pointColors[0];
					}
					for(int j = 1; j < pointColors.length; j++) {
						if(pointColorScale.length >= j) {
							if(pointColorScale[j - 1] > YValue) {
								break;
							}
							color = pointColors[j];
						}
					}
				}
				if(line.getIcon() == null) {
					graphics.fillRect(xs[i] - 3, ys[i] - 3, 6, 6, color);
				} else {
					graphics.drawImage(line.getIcon(), xs[i] - 4, ys[i] - 4);
				}
			}
			if(line.getValueFontData() != null) {
				YValue = ((Double)line.getElementY(i)).doubleValue();
				String txt = line.doubleToString(new Double(YValue));

				if(YValue == (int)YValue) {
					txt = new Integer((int)YValue).toString();
				}
				if(line.getLabelTemplate().length() > 0) {
					txt = line.getLabelTemplate();
				}
				if(line.getDataLabels() != null && line.getDataLabels().length > i) {
					txt = line.getDataLabels()[i];
				}

				Color color = systemColor(line.getValueColor());
				Font ontheflyFont = new Font(Display.getCurrent(), line.getValueFontData());
				graphics.drawText(txt, xs[i] + 7, ys[i], color, ontheflyFont);
				ontheflyFont.dispose();
			}
		}
		if(gridStyle != null) {
			double maxValues[] = new double[xs.length];
			double minValues[] = new double[20];
			if(minScaleFactors.length >= 1) {
				for(int j = 0; j < xs.length; j++) {
					minValues[j] = minScaleFactors[j];
				}
			}
			if(maxScaleFactors.length >= 1) {
				for(int j = 0; j < xs.length; j++) {
					maxValues[j] = maxScaleFactors[j];
				}
			}
			int tickInterval = 100 / scalingDivisions;
			double[] tickIntervalAbsValues = new double[xs.length];

			for(int j = 0; j < xs.length; j++) {
				tickIntervalAbsValues[j] = (maxValues[j] - minValues[j]) / scalingDivisions;
			}

			int tickAt = 0;
			double[] tickAtAbsValues = new double[xs.length];
			for(int j = 0; j < scalingDivisions; j++) {
				tickAt += tickInterval;
				for(int k = 0; k < xs.length; k++) {
					tickAtAbsValues[k] += tickIntervalAbsValues[k];
				}
				for(int i = 0; i < count; i++) {
					angle = 360.0D / count * i;

					angle += 90.0D;
					if(angle > 360.0D) {
						angle -= 360.0D;
					}
					int tmpradi = radi * tickAt / 100;
					double radian = 0.01745277777777778D * angle;
					double Sin = Math.sin(radian);
					double Cos = Math.cos(radian);
					int relativeY = (int)(Sin * (tmpradi / 2));
					int relativeX = (int)(Cos * (tmpradi / 2));
					relativeY *= -1;

					xs[i] = PieCenterX + relativeX;
					ys[i] = PieCenterY + relativeY;
				}
				if(sequenceNumber >= data.size() - 1) {
					gridStyle.drawPolygon(graphics, xs, ys, (int)count);
				}
				if(sequenceNumber >= data.size() - 1 && gridFontData != null) {
					double[] tickValues = new double[xs.length];
					String[] values = new String[xs.length];

					for(int i = 0; i < tickValues.length; i++) {
						tickValues[i] = tickAtAbsValues[i];
						values[i] = "" + tickValues[i];

						if(scalingLabelFormat.length > 0) {
							Object scalingLabel = scalingLabelFormat[i];
							DecimalFormat df = null;
							if(scalingLabel instanceof String) {
								df = new DecimalFormat((String)scalingLabelFormat[i]);
								values[i] = df.format(new Double(tickValues[i]));
							}
							if(scalingLabel instanceof Class<?>) {
								if(((Class<?>)scalingLabel).isEnum()) {
									try {
										values[i] =
												(String)enumConstants((Class<Enum>)scalingLabel)
														.get(kl++);
									} catch(Exception e) {
										e.printStackTrace();
									}
								}
							}
						}
					}

					Color color = systemColor(swtGridFontColor);
					Font ontheflyFont = new Font(Display.getCurrent(), gridFontData);
					if(markScalesOnEveryAxis) {
						for(int k = 0; k < xs.length; k++) {
							graphics.drawText("" + values[k],
									xs[k] - 3 - graphics.getFontWidth("" + values[k]), ys[k],
									color, ontheflyFont);
						}
					} else {
						graphics.drawText("" + values[0],
								xs[0] - 3 - graphics.getFontWidth("" + values[0]), ys[0], color,
								ontheflyFont);
					}
					ontheflyFont.dispose();
				}
			}
		}
	}

	/**
	 * @param axesFactors
	 *            the axesFactors to set
	 */
	public void setAxesFactors(String[] axesFactors) {
		this.axesFactors = axesFactors;
	}

	/**
	 * @param axisFactorColor
	 *            the axisFactorColor to set
	 */
	public void setAxisFactorColor(int swtAxisFactorColor) {
		this.swtAxisFactorColor = swtAxisFactorColor;
	}

	/**
	 * @param axisFactorColors
	 *            the axisFactorColors to set
	 */
	public void setAxisFactorColors(int[] swtAxisFactorColors) {
		this.swtAxisFactorColors = swtAxisFactorColors;
	}

	/**
	 * @param axisFactorFont
	 *            the axisFactorFont to set
	 */
	public void setAxisFactorFont(FontData axisFactorFontData) {
		this.axisFactorFontData = axisFactorFontData;
	}

	/**
	 * @param backStyle
	 *            the backStyle to set
	 */
	public void setBackStyle(FillStyle backStyle) {
		this.backStyle = backStyle;
	}

	/**
	 * @param border
	 *            the border to set
	 */
	public void setBorder(LineStyle border) {
		this.border = border;
	}

	/**
	 * @param chartRadius
	 *            the chartRadius to set
	 */
	public void setChartRadius(double chartRadius) {
		this.chartRadius = chartRadius;
	}

	/**
	 * @param drawCircle
	 *            the drawCircle to set
	 */
	public void setDrawCircle(boolean drawCircle) {
		this.drawCircle = drawCircle;
	}

	/**
	 * @param gridFont
	 *            the gridFont to set
	 */
	public void setGridFontData(FontData gridFontData) {
		this.gridFontData = gridFontData;
	}

	/**
	 * @param gridFontColor
	 *            the gridFontColor to set
	 */
	public void setGridFontColor(int swtGridFontColor) {
		this.swtGridFontColor = swtGridFontColor;
	}

	/**
	 * @param gridStyle
	 *            the gridStyle to set
	 */
	public void setGridStyle(LineStyle gridStyle) {
		this.gridStyle = gridStyle;
	}

	/**
	 * @param markScalesOnEveryAxis
	 *            the markScalesOnEveryAxis to set
	 */
	public void setMarkScalesOnEveryAxis(boolean markScalesOnEveryAxis) {
		this.markScalesOnEveryAxis = markScalesOnEveryAxis;
	}

	/**
	 * @param maxScaleFactors
	 *            the maxScaleFactors to set
	 */
	public void setMaxScaleFactors(double[] maxScaleFactors) {
		this.maxScaleFactors = maxScaleFactors;
	}

	/**
	 * @param minScaleFactors
	 *            the minScaleFactors to set
	 */
	public void setMinScaleFactors(double[] minScaleFactors) {
		this.minScaleFactors = minScaleFactors;
	}

	/**
	 * @param pointColors
	 *            the pointColors to set
	 */
	public void setPointColors(Color[] pointColors) {
		this.pointColors = pointColors;
	}

	/**
	 * @param pointColorScale
	 *            the pointColorScale to set
	 */
	public void setPointColorScale(double[] pointColorScale) {
		this.pointColorScale = pointColorScale;
	}

	/**
	 * @param scalingDivisions
	 *            the scalingDivisions to set
	 */
	public void setScalingDivisions(int scalingDivisions) {
		this.scalingDivisions = scalingDivisions;
	}

	/**
	 * @param scalingLabelFormat
	 *            the scalingLabelFormat to set
	 */
	public void setScalingLabelFormat(Object... scalingLabelFormat) {
		this.scalingLabelFormat = Arrays.asList(scalingLabelFormat).toArray();
	}

	/** */
	public void use(SpiderChart model) {
		requireNonNull(model);
		maxScaleFactors = model.maxScales();
		minScaleFactors = model.minScales();
		axesFactors = model.axesNames();
		scalingLabelFormat = model.axesScalingLabelFormats();
	}

	private Color systemColor(int swtColor) {
		return Display.getCurrent().getSystemColor(swtColor);
	}

	/** Disposes the plotter's resources. */
	public void dispose() {
		for(DataSequence ds : data) {
			ds.dispose();
		}
	}

	/** plots the values */
	public void plot(SpiderChartSwtGraphics g) {
		for(int i = 0; i < data.size(); i++) {
			plotSequence(g, data.get(i), i);
		}
	}

	/**
	 * plots the background of the plotting canvas
	 */
	public void plotBackground(SpiderChartSwtGraphics graphics) {
		if(getBackgroundFillStyle() != null) {
			getBackgroundFillStyle().draw(graphics, x, y, x + getWidth(), y + getHeight());
		}
		if(backgroundImage != null) {
			final int w = backgroundImage.getWidth();
			final int h = backgroundImage.getHeight();
			if(w > -1 && h > -1) {
				int toCenterX = (getWidth() - w) / 2;
				if(toCenterX < 0) {
					toCenterX = 0;
				}
				int toCenterY = (getHeight() - h) / 2;
				if(toCenterY < 0) {
					toCenterY = 0;
				}
				graphics.drawImage(backgroundImage, toCenterX + x, y + toCenterY);
			}
		}
	}

	/** Returns depth. */
	public int getDepth() {
		return depth;
	}

	/** Returns visible width. */
	public int getVisibleWidth() {
		return visibleWidth;
	}

	/** Sets visible width. */
	public void setVisibleWidth(int visibleWidth) {
		this.visibleWidth = visibleWidth;
	}

	/** Returns visible height. */
	public int getVisibleHeight() {
		return visibleHeight;
	}

	/** Sets visible height. */
	public void setVisibleHeight(int visibleHeight) {
		this.visibleHeight = visibleHeight;
	}

	/** Plots the axes of the spider chart. */
	public void drawAxes(SpiderChartSwtGraphics graphics) {
		List<Axis> axes = chart.getAxes();
		int size = axes.size();
		double segmentDegree = 360.0 / size;
		double angleDegree = 90.0;

		int halfWidth = getWidth() / 2;
		int halfHeight = getHeight() / 2;
		int centerX = getX() + halfWidth;
		int centerY = getY() + halfHeight;

		int wExtent = (int)(relativeMargin * halfWidth);
		int hExtent = (int)(relativeMargin * halfHeight);
		int extent = Math.min(wExtent, hExtent);

		for(int i = 0; i < size; i++) {
			Axis axis = axes.get(i);

			// draw line
			double angleRad = Math.PI * angleDegree / 180.0;
			int outerX = centerX + (int)(Math.cos(angleRad) * extent);
			int outerY = centerY + (int)(Math.sin(angleRad) * extent);
			AxisStyle style = axisStyle.get(axis);
			style.getLineStyle().draw(graphics, centerX, centerY, outerX, outerY);

			// draw label
			drawAxisLabel(graphics, axis, style, centerX, centerY, outerX, outerY);

			// TODO: draw segment labels

			angleDegree = (angleDegree + segmentDegree) % 360.0;
		}
	}

	/** Draws the axis label at the correct position. */
	private void drawAxisLabel(SpiderChartSwtGraphics graphics, Axis axis, AxisStyle style,
			int centerX, int centerY, int outerX, int outerY) {
		String axisLabel = axis.getName();
		FontData fontData = style.getLabelStyle().getFontData();
		GC gc = graphics.getGraphics();

		Font font = new Font(Display.getCurrent(), fontData);
		Font oldFont = gc.getFont();
		gc.setFont(font);
		Color oldColor = gc.getForeground();
		gc.setForeground(Display.getCurrent().getSystemColor(style.getLabelStyle().getSwtColor()));

		int textExtent = graphics.getGraphics().textExtent(axisLabel).x;
		if(outerX > centerX) {
			outerX += 5;
		} else if(outerX < centerX) {
			outerX = outerX - 5 - textExtent;
		} else {
			outerX = outerX - textExtent / 2;
		}
		if(outerY > centerY) {
			outerY = outerY + 5;
		} else if(outerY < centerY) {
			outerY = outerY - 5 - fontData.getHeight();
		} else {
			outerY = outerY - fontData.getHeight() / 2;
		}
		gc.drawString(axisLabel, outerX, outerY, true);

		gc.setFont(oldFont);
		gc.setForeground(oldColor);
		font.dispose();
	}

	/** Sets the style for the given axis. */
	public void setAxisStyle(Axis axis, AxisStyle style) {
		axisStyle.put(axis, style);
	}

	/** Sets the style for the given data series. */
	public void setDataSeriesStyle(DataSeries dataSeries, LineStyle style) {
		dataSeriesStyle.put(dataSeries, style);
	}
}
