/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.plotter;

import static java.util.Objects.requireNonNull;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.fortiss.tooling.spiderchart.gc.swt.SpiderChartSwtGraphics;
import org.fortiss.tooling.spiderchart.model.SpiderChart;

/**
 * Class representing the spider charts title.
 *
 * @author mondal
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: D822418216774A7766E735ECD40429CA
 */
public final class SpiderChartTitle extends SpiderChartComponentBase {

	/** The title text. */
	private String text;

	/** Constructor */
	public SpiderChartTitle(String text, SpiderChart chart) {
		super(chart);
		this.text = requireNonNull(text);
		setSwtForegroundColor(SWT.COLOR_BLACK);
		setFontName("Verdana", 14, SWT.NORMAL);
	}

	/** Used to draw the title */
	public void draw(SpiderChartSwtGraphics g) {
		if(text == null) {
			return;
		}
		GC gc = g.getGraphics();
		Font ontheflyFont = createFont();

		Font oldFont = gc.getFont();
		Color oldColor = gc.getForeground();

		gc.setFont(ontheflyFont);
		int toCenterX = (getWidth() + gc.textExtent(text).x) / 2;
		int toCenterY = (getHeight() + getFontData().getHeight()) / 2;

		gc.drawString(text, getX() + toCenterX, getY() + toCenterY + g.getFontHeight(), true);

		gc.setForeground(oldColor);
		gc.setFont(oldFont);
		ontheflyFont.dispose();
	}

	/** Sets the chart title text. */
	public final void setText(final String text) {
		this.text = text;
	}
}
