/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.swt;

import static org.eclipse.swt.SWT.CTRL;
import static org.fortiss.tooling.spiderchart.gc.AbstractGraphicsSupplier.getGraphics;
import static org.fortiss.tooling.spiderchart.gc.AbstractGraphicsSupplier.startUiThread;

import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.fortiss.tooling.spiderchart.SpiderChartOld;
import org.fortiss.tooling.spiderchart.gc.swt.SpiderChartSwtGraphics;
import org.fortiss.tooling.spiderchart.listener.ISpiderChartListener;

/**
 * Represents a canvas on which chart will be displayed
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class SpiderChartCanvas extends Canvas implements ISpiderChartListener {

	/** Actual Spider Chart Ref */
	private SpiderChartOld chart = null;

	/** Constructor */
	public SpiderChartCanvas(final Composite parent, final int style) {
		super(parent, style | CTRL);

		this.addPaintListener(e -> SpiderChartCanvas.this.paintChart(e));
		final MouseMoveListener mouseMove = e -> SpiderChartCanvas.this.mouseMoved(e);
		this.addMouseMoveListener(mouseMove);

		final MouseAdapter mouseAdapter = new MouseAdapter() {
			/** {@inheritDoc} */
			@Override
			public void mouseDown(final MouseEvent e) {
				SpiderChartCanvas.this.mouseClick();
			}
		};
		this.addMouseListener(mouseAdapter);

		this.addControlListener(new ControlListener() {
			/** {@inheritDoc} */
			@Override
			public void controlMoved(final ControlEvent e) {
				// Not used
			}

			/** {@inheritDoc} */
			@Override
			public void controlResized(final ControlEvent e) {
				SpiderChartCanvas.this.resizeChart();
			}
		});
	}

	/** Returns the spider chart */
	public SpiderChartOld getChart() {
		return this.chart;
	}

	/** Mouse Click event */
	private void mouseClick() {
		if(this.chart != null) {
			this.chart.mouseClick();
		}
	}

	/** Mouse Moved Event */
	private void mouseMoved(final MouseEvent e) {
		if(this.chart != null) {
			this.chart.mouseMoved(e.x, e.y);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void onChartEvent(final SpiderChartOld c, final int type) {
		if(type == 4) {
			this.redraw();
		}
		if(type == 1) {
			startUiThread(() -> {
				if(!SpiderChartCanvas.this.isDisposed()) {
					SpiderChartCanvas.this.redraw();
				}
			});
		}
	}

	/** {@inheritDoc} */
	@Override
	public void onPaintUserExit(SpiderChartOld c, SpiderChartSwtGraphics g) {
		// Not used
	}

	/** Draws the Spider Chart */
	protected void paintChart(final PaintEvent e) {
		try {
			this.resizeChart();
			SpiderChartSwtGraphics g = getGraphics(e.gc);
			this.chart.paint(g);
			g.dispose();
		} catch(final Exception err) {
			err.printStackTrace();
		}
	}

	/** Resizes the spider chart */
	protected void resizeChart() {
		this.chart.setWidth(this.getSize().x + 1);
		this.chart.setHeight(this.getSize().y + 1);
	}

	/** Setter for Spider Chart */
	public void setChart(final SpiderChartOld c) {
		if(this.chart != null) {
			this.chart.removeChartListener(this);
		}
		this.chart = c;
		this.chart.addChartListener(this);
	}
}
