/*--------------------------------------------------------------------------+
$Id: SpiderChartSwtColor.java 17058 2016-05-24 09:30:52Z hoelzl $
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.util;

import static java.lang.Math.max;
import static java.lang.Math.min;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;

/**
 * Utility methods for creating brighter and darker colors.
 *
 * @author mondal
 * @author $Author: hoelzl $
 * @version $Rev: 17058 $
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class ColorUtil {
	/** Returns the color, which is 10% brighter than the given color. */
	public static Color getBrighterColor(Color color) {
		int percent = 10;
		int rr = color.getRed();
		int gg = color.getGreen();
		int bb = color.getBlue();
		int r = rr + percent * (rr / 100);
		int g = gg + percent * (gg / 100);
		int b = bb + percent * (bb / 100);
		return new Color(Display.getCurrent(), min(r, 255), min(g, 255), min(b, 255));
	}

	/** Returns the color, which is 10% darker than the given color. */
	public static Color getDarkerColor(Color color) {
		int percent = 10;
		int rr = color.getRed();
		int gg = color.getGreen();
		int bb = color.getBlue();
		int r = rr - percent * (rr / 100);
		int g = gg - percent * (gg / 100);
		int b = bb - percent * (bb / 100);

		return new Color(Display.getCurrent(), max(r, 0), max(g, 0), max(b, 0));
	}
}
